@@ -1,6 +1,6 @@
 /***********************************************************************
  *
- * Copyright (C) 2010-2011 Innocent De Marchi <tangram.peces@gmail.com>
+ * Copyright (C) 2010-2013 Innocent De Marchi <tangram.peces@gmail.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,9 +29,16 @@ int main(int argc, char *argv[])
 {
     QApplication app(argc, argv);
         app.setApplicationName("Solitari");
-        app.setApplicationVersion("1.2");
+        app.setApplicationVersion("2.0");
         app.setOrganizationName("De Marchi");
 
+        //Això pels accents a les cadenes
+        QTextCodec *linuxCodec = QTextCodec::codecForName("UTF-8");
+        QTextCodec::setCodecForTr(linuxCodec);
+        QTextCodec::setCodecForCStrings(linuxCodec);
+        QTextCodec::setCodecForLocale(linuxCodec);
+
+
     //Això per a que surtin traduïdes les cadenes dels formularis predefinits Qt
      QString qt_translatorFileName = QLatin1String("qt_");
      qt_translatorFileName += QLocale::system().name();
@@ -40,7 +47,7 @@ int main(int argc, char *argv[])
      if (qt_translator->load(qt_translatorFileName, QLibraryInfo::location(QLibraryInfo::TranslationsPath)))
          app.installTranslator(qt_translator);
 
-    //Això per les traduccions
+   //Això per les traduccions
    appTranslator= new QTranslator();
    appTranslator->load("solitari_" + QLocale::system().name(), app.applicationDirPath()+QDir().separator()+"locales");
 
