@@ -1,6 +1,6 @@
 /***********************************************************************
  *
- * Copyright (C) 2010-2011 Innocent De Marchi <tangram.peces@gmail.com>
+ * Copyright (C) 2010-2013 Innocent De Marchi <tangram.peces@gmail.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -53,11 +53,15 @@ Tauler::Tauler(  QUndoStack *movimentsUndoStack , Rellotge *rellotge,frmPrincipa
         setFrameStyle(QFrame::NoFrame);
         setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
         setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-        setMinimumSize(QSize(300,300));
+        //setMinimumSize(frmPrincipal->minimumSize());
+        setMinimumSize(QSize(290,290));
 
         comptadorFitxesJoc=0;
+        comptadorFitxesEstatZeroJoc=0;
         //inicialment es juga
         p_estatDelJoc=joc;
+        p_SolitariAtzar=false;
+        p_SolitariPersonalitzat=false;
 }
 
 /*
@@ -65,7 +69,7 @@ Tauler::Tauler(  QUndoStack *movimentsUndoStack , Rellotge *rellotge,frmPrincipa
   */
 int Tauler::estatFitxaJoc( const  QPoint& coordenades) const {
     if (m_fitxes.contains(coordenades.x()*100+coordenades.y())){
-       return   m_fitxes.value(coordenades.x()*100+coordenades.y())->estat();
+       return m_fitxes.value(coordenades.x()*100+coordenades.y())->estat();
            }
    else return -1;
 }
@@ -82,6 +86,7 @@ void Tauler::marcaMoviment (QString moviment){
     m_fitxes.value(moviments.value(0).toInt())->setEstat(2);
     m_fitxes.value(moviments.value(2).toInt())->setEstat(3);
 }
+
 /*
   Elimina els buits marcats com a final d'un moviment anterior
   */
@@ -99,23 +104,31 @@ void Tauler::eliminaMarquesMoviments(const  QPoint& coordenades){
 }
 
 
-void Tauler::ferMoviment (int fitxaInicial, int fitxaBotada, int fitxaFinal){
+void Tauler::ferMoviment (int fitxaInicial, int fitxaBotada, int fitxaFinal,bool missatge){
        Moviment* nouMoviment = new Moviment(fitxaInicial,fitxaBotada,fitxaFinal,p_tipusMoviment,this);
+       /*qCritical("%s", qPrintable(QString("%1 %2 %3").
+                                  arg(fitxaInicial).
+                                  arg(fitxaBotada).
+                                  arg(fitxaFinal)));*/
        p_movimentsUndoStack->push(nouMoviment);
        p_frmPrinci->iniciSolucioAction->setEnabled(p_movimentsUndoStack->canUndo());
-       if(p_estatDelJoc==joc){
+       //NO es mostrarà el missatge en els solitaris
+       //a l'atzar
+       if( (p_estatDelJoc==joc) && missatge){
            p_frmPrinci->mostraMissatge(3);}
          //  p_frmPrinci->statusBar()->showMessage(missatgeNumeroMoviments());;}
-       marcaFinalDeJoc(p_tipusMoviment);
+       marcaFinalDeJoc(p_tipusMoviment,true);
+       ocuparFitxesFinalJoc();
 }
 
-void Tauler::marcaFinalDeJoc(int direccioMoviment){
+void Tauler::marcaFinalDeJoc(int direccioMoviment, bool marcaFinal){
     if( jocFinalitzat()){
         QHashIterator <int, Fitxa*> i(m_fitxes);
         int numFitxes=0, ordFitxa=0;
+        int numFitxesFinalJoc=0;
         jocResolt=1;
-        p_rellotge->aturaRellotge();
-        if(p_estatDelJoc==joc){
+        if(p_estatDelJoc==joc){            
+            p_rellotge->aturaRellotge();
             p_frmPrinci->mostraMissatge(0);}
         //es posen les fitxes en color vermell
         while (i.hasNext()) {
@@ -124,22 +137,23 @@ void Tauler::marcaFinalDeJoc(int direccioMoviment){
                   i.value()->setEstat(5);
                   numFitxes++;
                   ordFitxa=i.key();
+                  if(i.value()->fitxaFinalJoc()){
+                   numFitxesFinalJoc++;
+                  }
               }          
-    }
-
-        //En les modalitats "inverses" no queda només una fitxa
-        if ( (numFitxes==1) && (direccioMoviment != 2)){
-          m_fitxes.value(ordFitxa)->setEstat(4);
-          p_frmPrinci->mostraMissatge(1);
-          jocResolt=2;
-          p_rellotge->aturaRellotge();
-          if(p_estatDelJoc==joc){
-              controlaRecords();
-              p_frmPrinci->desaSolucions1();
-              p_frmPrinci->veureSolucionsSlot();}
         }
 
-        if (  (numFitxes==numeroDeMovimentsJoc()+1) && (direccioMoviment == 2)){
+        //En les modalitats "inverses" no queda només una fitxa
+        //Aqui es controla quan queda només una fitxa o
+        //bé els casos en què hi ha fitxes marcades
+        //com a final del joc
+        if ( ( (numFitxes==1) && (direccioMoviment != 2)
+               && comptadorFitxesFinalJoc==0 )
+            || (nomesQuedenFitxesMarcadesFinalJoc() && (direccioMoviment != 2) )
+                                    ) {
+          jocResolt=2;
+          p_rellotge->aturaRellotge();
+          m_fitxes.value(ordFitxa)->setEstat(4);
           i.toFront();
           while (i.hasNext()) {
               i.next();
@@ -148,66 +162,174 @@ void Tauler::marcaFinalDeJoc(int direccioMoviment){
               }
           }
           p_frmPrinci->mostraMissatge(1);
-          jocResolt=2;
-          p_rellotge->aturaRellotge();
           if(p_estatDelJoc==joc){
               controlaRecords();
               p_frmPrinci->desaSolucions1();
               p_frmPrinci->veureSolucionsSlot();}
         }
-      setInteractive(false);
-    }
+
+        /* La primera opció és pels jocs que començan amb una fitxa.
+         * La segona, per aquells que començan amb més d'una fitxa
+         * (i això és per a les modaltats inverses)
+        */
+          if ( ((numFitxes==numeroDeMovimentsJoc()+1) && (direccioMoviment == 2)
+              && comptadorFitxesJoc==1)  ||
+             ((numFitxes==numeroDeMovimentsJoc()+comptadorFitxesJoc) &&
+             (direccioMoviment == 2)) ){
+          i.toFront();
+          while (i.hasNext()) {
+              i.next();
+              if (i.value()->estat()==5){
+                  i.value()->setEstat(4);
+              }
+          }
+          p_rellotge->aturaRellotge();
+          jocResolt=2;
+          p_frmPrinci->mostraMissatge(1);
+          if(p_estatDelJoc==joc){
+              controlaRecords();
+              p_frmPrinci->desaSolucions1();
+              p_frmPrinci->veureSolucionsSlot();}
+        }
+     //14/12/12 Evita que quedin les fitxes bloquejades quan NO es
+     //resol el joc
+     //03/01/12  què????
+     //04/01/12 S'ha deixat que quedin bloquejades les fitxes quan
+     //s'hagi resolt el joc
+     //if (jocResolt==2){
+       //  setInteractive(false);}
+
+     /*si es tracta d'un solitari personalitzat,
+      *cal comprovar si s'ha de desar
+      */
+     if( (solitariPersonalitzat()) && (jocResolt==2) &&
+         (p_estatDelJoc==joc) && marcaFinal){
+         p_frmPrinci->solitariPersonalitzat_desarSolitari();
+     }
+
+    }// final if jocResolt()
+
 }
 
+/*
+ * Comprova si només queden fitxes marcades com a final
+ * de joc (i estan activades!)
+ */
+bool Tauler::nomesQuedenFitxesMarcadesFinalJoc(){
+  QHashIterator <int, Fitxa*> i(m_fitxes);
+  bool nomesFinalJoc=true;
+  while (i.hasNext()) {
+        i.next();
+       // if ( ( ( (i.value()->estat()==1) || (i.value()->estat()==5) ) &&
+        if ( (  (i.value()->estat() != 0) &&
+             (!i.value()->fitxaFinalJoc())
+                                   ) ||
+            ((i.value()->estat()==0) &&
+                (i.value()->fitxaFinalJoc()))
+                                            ) {
+             nomesFinalJoc=false;
+       }
+   }
+  return nomesFinalJoc;
+  }
+
 /*
   Comprova, desprès de cada moviment, si el joc s'ha acabat
+  El joc s'ha acabat si només queda una fitxa (o totes en
+  les modalitats inverses)
+  si només queden les fitxes (ocupades) marcades
+  com a final de joc
+  o si les fitxes que queden no son final de joc
+  i no es poden moure
   */
 bool Tauler::jocFinalitzat(){
+
    QHashIterator <int, Fitxa*> i(m_fitxes);
+   //Pressuposam que el joc ha acabat
    bool finalitzat=true;
    while (i.hasNext()) {
         i.next();
-        if (i.value()->estat()==1){
-           if ( i.value()->esPotMoure()){finalitzat=false;}
+        //Només comprovam les fitxes actives!
+        if ( (i.value()->estat()==1) ){
+            /* Si la fitxa es pot moure i
+             * no està marcada com a final
+             * del joc-> el joc no ha acabat
+             */
+            if  (i.value()->esPotMoure() &&
+                !i.value()->fitxaFinalJoc()){
+                finalitzat=false;
+                break;
+             }
+
+            /* La fitxa es pot moure, però està marcada com
+             * a final de joc i queden altres fitxes que
+             * no estan marcades com a final de joc
+             */
+            if (
+                 i.value()->esPotMoure() &&
+                 i.value()->fitxaFinalJoc() &&
+                 !nomesQuedenFitxesMarcadesFinalJoc()
+                                   ){
+                finalitzat=false;
+                break;
+                }
+            if(p_tipusMoviment==2 && i.value()->esPotMoure()){
+                finalitzat=false;
+                break;
+            }
+            if( i.value()->fitxaFinalJoc() &&
+                !nomesQuedenFitxesMarcadesFinalJoc()){
+              finalitzat=false;
+              break;
+            }
         }
     }
  return finalitzat;
 }
 
 void Tauler::controlaRecords(){
+    //dels solitaris especials no es desen els records
+    if( !esSolitariEspecial()){
     QSettings records;
     QString record=records.value(QString("%1/record").arg(p_modalitatJocActual),QString("100:00:00")).toString();
     if (p_rellotge->retornaTemps(1)< record){
         QSettings().setValue(QString("%1/record").arg(p_modalitatJocActual), p_rellotge->retornaTemps(1));
         //missatge nou record personal
         p_frmPrinci->mostraMissatge(2);
-     //   QMessageBox::information(this,tr("Nova marca personal"),
-       //                          tr("Heu establert un nou record personal en aquesta modalitat de joc"));
     }
+  }
+  //  else qDebug("és una solitari especial! %d",p_modalitatJocActual.toInt());
 }
 
 /*
 Carrega les fitxes de la modalitat de joc passada per paràmetre
   */
 void Tauler::joc_CarregaJoc( QString p_tipusJoc, QString p_filesColumnes, QString p_estatFitxes){
+    /*
+     *Això fa que es «redimensioni» correctament el tauler
+     *de joc quan es canvia de modalitat
+     */
     scene()->clear();
     QGraphicsScene* sceneJoc1 = new QGraphicsScene(this);
-     setScene(sceneJoc1);
-    //sceneJoc->clear();
+    setScene(sceneJoc1);
+
     p_rellotge->aturaRellotge();
     p_rellotge->estableixTemps(QString("00:00:00"));
     comptadorFitxesJoc=0;
+    comptadorFitxesEstatZeroJoc=0;
     p_estatDelJoc=joc;
 
 
     QStringList  tipusJoc = p_tipusJoc.split(" ");
-    p_modalitatJocActual=tipusJoc.value(0);
+    setModalitatJocActual(tipusJoc.value(0));
+    //Comprovam els solitaris especials (atzar i personalitzats)
+    //setSolitariEspecial(p_modalitatJocActual);
     QStringList  filesColumnes= p_filesColumnes.split(" ");
     QStringList  estatFitxes = p_estatFitxes.split(" ");
     p_tipusMoviment=tipusJoc.value(1).toInt();
     p_numFitxesAillades=tipusJoc.value(2).toInt();
     p_numMoviments=tipusJoc.value(3).toInt();
-    if (p_tipusMoviment==2){p_estatFitxaMitjana=0;}
+    if ( (p_tipusMoviment==2) || (p_tipusMoviment==5) ){p_estatFitxaMitjana=0;}
     else {p_estatFitxaMitjana=1;}
 
     int files, columnes;
@@ -222,12 +344,14 @@ void Tauler::joc_CarregaJoc( QString p_tipusJoc, QString p_filesColumnes, QStrin
                        Fitxa *fitxa= new Fitxa(this);
                        fitxa->setCoordenades(coord);
                        fitxa->setEstat(estatFitxes.value(j*columnes+i).toInt());
+                       //fitxa->setFitxaSolitariPersonalitzat(false);
                        fitxa->setTipusMoviment(tipusJoc.value(1).toInt());
                        fitxa->setPos(fitxa->coordenades().y()*distanciaEntreFitxes*30,
                                              fitxa->coordenades().x()*distanciaEntreFitxes*30);
                        m_fitxes.insert(coord.x()*100+coord.y(),fitxa);
                        scene()->addItem(fitxa);
                        if (fitxa->estat()==1) {++comptadorFitxesJoc;}
+                       if (fitxa->estat()==0){++comptadorFitxesEstatZeroJoc;}
                    }//if estatFitxes
                } //for columnes
            }//for files
@@ -254,6 +378,7 @@ void Tauler::joc_CarregaJoc( QString p_tipusJoc, QString p_filesColumnes, QStrin
                        m_fitxes.insert(coord.x()*100+coord.y(),fitxa);
                        scene()->addItem(fitxa);
                        if (fitxa->estat() == 1) {++comptadorFitxesJoc;}
+                       if (fitxa->estat()==0){++comptadorFitxesEstatZeroJoc;}
                         }//if estat fitxa
                        ++contador;
                } //for columnes
@@ -263,9 +388,77 @@ p_movimentsUndoStack->clear();
 setInteractive(true);
 //Això per a què s'ajusti a la finestra
 fitInView(sceneRect(), Qt::KeepAspectRatio);
-//  QVector<QVector<int> > matriuJoc(7,QVector<int>(7));
 }
 
+/*Controla els tipus de joc especials
+ */
+void Tauler::setSolitariEspecial(QString codiJoc){
+  //pressuposam que no és un solitari especial
+  setSolitariAtzar(false);
+  setSolitariPersonalitzat(false);
+  //Marcam els jocs a l'atzar
+  setSolitariAtzar(codiJoc=="1000");
+  //marcam els jocs personalitzats
+  setSolitariPersonalitzat(
+   codiJoc=="1001" || codiJoc=="1002" ||codiJoc=="1003" );
+}
+
+/*
+ * En els jocs amb fitxes finals marcades,
+ * controla si el número de moviments per acabar
+ * el joc permet deixar les fitxes finals ocupades
+ */
+bool Tauler::ocuparFitxesFinalJoc(){
+  bool resultat=true;
+  /* Si és un joc amb fitxes marcades com a final del joc
+   */
+  QHashIterator <int, Fitxa*> fitxesFinalsJoc(m_fitxes);
+  bool ambFitxesFinalJoc=false;
+  while (fitxesFinalsJoc.hasNext()) {
+       fitxesFinalsJoc.next();
+       if ( (fitxesFinalsJoc.value()->fitxaFinalJoc()) ){
+         ambFitxesFinalJoc=true;
+       }
+  }
+  if(ambFitxesFinalJoc){
+   resultat=( (numeroFitxesFinalsSenseOcupar()<4) &&
+              (numeroFitxesPerEliminar()>numeroFitxesFinalsSenseOcupar()) );
+  }
+  return resultat;
+}
+
+/* Retorna el número de fitxes marcades com a final
+ * del joc que estan sense ocupar
+ */
+int Tauler::numeroFitxesFinalsSenseOcupar(){
+    QHashIterator <int, Fitxa*> fitxesFinalsJoc(m_fitxes);
+    int numeroFitxesFinalJoc=0;
+    while (fitxesFinalsJoc.hasNext()) {
+         fitxesFinalsJoc.next();
+         if ( (fitxesFinalsJoc.value()->fitxaFinalJoc()) &&
+             fitxesFinalsJoc.value()->estat()==0){
+           numeroFitxesFinalJoc++;
+         }
+    }
+   return numeroFitxesFinalJoc;
+}
+
+/* Calcula quantes fitxes cal eliminar per acabar
+ * el joc (fitxes actives que no estan marcades com
+ * a final de joc)
+ */
+int Tauler::numeroFitxesPerEliminar(){
+    QHashIterator <int, Fitxa*> fitxesFinalsJoc(m_fitxes);
+    int numeroFitxesFinalJoc=0;
+    while (fitxesFinalsJoc.hasNext()) {
+         fitxesFinalsJoc.next();
+         if ( !(fitxesFinalsJoc.value()->fitxaFinalJoc()) &&
+             fitxesFinalsJoc.value()->estat()==1){
+           numeroFitxesFinalJoc++;
+         }
+    }
+   return numeroFitxesFinalJoc;
+}
 
  void Tauler::resizeEvent(QResizeEvent* event) {
         fitInView(sceneRect(), Qt::KeepAspectRatio);
@@ -286,7 +479,13 @@ fitInView(sceneRect(), Qt::KeepAspectRatio);
      }
  }
 
- QStringList Tauler::solucionaJocActual_CarregaMovimentsArbre(QStringList movimentsFets, QString darrerMoviment){
+
+ /* Comprova els moviments possibles, els selecciona segons diversos
+  * criteris i preferències, i retorna una llista amb els moviments
+  * preferits
+  */
+ QStringList Tauler::solucionaJocActual_CarregaMovimentsArbre
+                     (QStringList movimentsFets, QString darrerMoviment){
 
     QHashIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
     QStringList itemsArbre; //tots els moviments possibles
@@ -295,7 +494,7 @@ fitInView(sceneRect(), Qt::KeepAspectRatio);
     QStringList itemsArbre3;//moviments per preferència que enllaçan amb l'anterior
     QStringList itemsArbre4;//moviments per nivell d'aïllament de les fitxes
     QStringList itemsArbre5;//moviments pel número de moviments posteriors
-    QStringList itemsArbre6;//moviments pel número de moviments posteriors
+    QStringList itemsArbre6;//moviments que cumpleixen almenys 3 requisits
 
   //  solucionaJocActual_FitxesAillades();
     int numeroMovimentsInicialsAtzar=3;
@@ -304,22 +503,29 @@ fitInView(sceneRect(), Qt::KeepAspectRatio);
       while (iterator_fitxes.hasNext()) {
         iterator_fitxes.next();
 
-        int index_fitxaInicial=iterator_fitxes.value()->coordenades().x()*100+iterator_fitxes.value()->coordenades().y();
+        int index_fitxaInicial=
+            iterator_fitxes.value()->coordenades().x()*100+
+            iterator_fitxes.value()->coordenades().y();
         //només comprovam les fitxes actives
        if ( m_fitxes.value(index_fitxaInicial)->estat()==1){
         int movimentsPosibles=0;
+        //qDebug("movimentsPossibles.count() %d",iterator_fitxes.value()->movimentsPossibles.count());
         for(int j=0 ; j<iterator_fitxes.value()->movimentsPossibles.count();++j){
             int index_fitxaBotada=iterator_fitxes.value()->movimentsPossibles.at(j).x();
             int index_fitxaFinal=iterator_fitxes.value()->movimentsPossibles.at(j).y();
+            //qDebug("p_estatFitxaMitjana %d ",p_estatFitxaMitjana);
             if (   (m_fitxes.value(index_fitxaBotada)->estat()==p_estatFitxaMitjana) &&
                 (m_fitxes.value(index_fitxaFinal)->estat()==0) ){
                 QString moviment=QString("%1 %2 %3").arg(
                         index_fitxaInicial).arg(
                         index_fitxaBotada).arg(
                         index_fitxaFinal);
-                //ës possible aquest moviment
+               //és possible aquest moviment
               if ( (movimentsFets.indexOf(QString("%1 %2").arg(p_movimentsUndoStack->index()+1).
-                                                 arg(moviment))==-1) && (solucionaJocActual_CalculaMovimentsPosteriors(moviment)) ){
+                   arg(moviment))==-1)
+                 //Aquesta condició no és eficient 19/12/11
+                 // && (solucionaJocActual_CalculaMovimentsPosteriors(moviment))
+                   ){
                 itemsArbre.append(moviment);
                 if (p_movimentsUndoStack->index()>numeroMovimentsInicialsAtzar+10){return itemsArbre;}
                 ++movimentsPosibles;
@@ -336,7 +542,7 @@ fitInView(sceneRect(), Qt::KeepAspectRatio);
       //Miram quins moviments maximitzen el número de moviments posteriors
       itemsArbre5=solucionaJocActual_CalculaNumeroMovimentsPosteriors(itemsArbre);
 
-      //MIram si és possible enllaçar algun dels moviments amb el darrerMoviment
+      //Miram si és possible enllaçar algun dels moviments amb el darrerMoviment
      QStringList fitxesDarrerMoviment=QStringList(darrerMoviment.split(" "));
      if (fitxesDarrerMoviment.count()>0){
    //  QString fitxaFinalDarrerMoviment=fitxesDarrerMoviment.value(fitxesDarrerMoviment.count()-1);
@@ -380,7 +586,8 @@ for(int j=0 ; j<itemsArbre5.count();++j){
     if (repeticio>2){itemsArbre6.append(itemsArbre5.value(j));}
 }
 
-      // moviments que cumpleixen més de 3 requisits
+QStringList movimentsSelecionats;
+/*      // moviments que cumpleixen més de 3 requisits
       if (itemsArbre6.count()>0){return itemsArbre6;}
       else if (itemsArbre3.count()>0){return itemsArbre3;}
       //Maximitzar el número de moviments
@@ -389,8 +596,43 @@ for(int j=0 ; j<itemsArbre5.count();++j){
       else  if (itemsArbre1.count()>0){return itemsArbre1;}
       // preferència en el moviment
       else if (itemsArbre2.count()>0){return itemsArbre2;}
-      else return itemsArbre;
-  }
+      else return itemsArbre;*/
+
+// moviments que cumpleixen més de 3 requisits
+if (itemsArbre6.count()>0){movimentsSelecionats.append(itemsArbre6);}
+else if (itemsArbre3.count()>0){movimentsSelecionats.append(itemsArbre3);}
+//Maximitzar el número de moviments
+else  if (itemsArbre5.count()>0){movimentsSelecionats.append(itemsArbre5);}
+// nomès enllaçar moviment
+else  if (itemsArbre1.count()>0){movimentsSelecionats.append(itemsArbre1);}
+// preferència en el moviment
+else if (itemsArbre2.count()>0){movimentsSelecionats.append(itemsArbre2);}
+else movimentsSelecionats.append(itemsArbre);
+
+//Ara comprova si hi ha algun moviment que te com a final un fitxa
+//marcada com a final de joc
+if(comptadorFitxesFinalJoc>0){
+   // qDebug("movimentsSelecionats.count() %d",movimentsSelecionats.size());
+ QStringList movimentsAmbFitxaFinalJoc;
+ for(int v=0 ; v<movimentsSelecionats.count();++v){
+   //qDebug("EN EL FOR");
+   QStringList fitxes=
+           QStringList(movimentsSelecionats.value(v).split(" "));
+   if (m_fitxes.value(
+       fitxes.value(2).toInt())->fitxaFinalJoc()){
+       //qDebug("FITXA FINAL!");
+       movimentsAmbFitxaFinalJoc.append(movimentsSelecionats.value(v));
+   }
+ }
+ if(movimentsAmbFitxaFinalJoc.count()>0){
+     //qDebug("*** hi ha moviments finals");
+   movimentsSelecionats.clear();
+   movimentsSelecionats.append(movimentsAmbFitxaFinalJoc);
+ }
+ //else qDebug("no hi ha moviments finals");
+}
+ return movimentsSelecionats;
+}
 
  /*
    Per a cada un dels moviments passats, calcula el nombre de moviments posibles
@@ -404,7 +646,7 @@ for(int j=0 ; j<itemsArbre5.count();++j){
      for(int j=0 ; j<movimentsPosibles.count();++j){
          //Primer "simulam" el moviment
          QStringList moviment=QStringList(movimentsPosibles.value(j).split(" "));
-         if (p_tipusMoviment != 2){
+         if ( (p_tipusMoviment != 2) && (p_tipusMoviment != 5)){
              m_fitxes.value(moviment.value(0).toInt())->setEstat(0);
              m_fitxes.value(moviment.value(1).toInt())->setEstat(0);
              m_fitxes.value(moviment.value(2).toInt())->setEstat(1);
@@ -440,20 +682,46 @@ for(int j=0 ; j<itemsArbre5.count();++j){
   return llistaMovimentsMaxims;
  }
 
+ /* Comprova si s'ha arribat al final del joc. Això
+  * és molt útil en les modalitats que tenen el final
+  * de joc marcat
+  */
+bool Tauler::arribatFinalJoc(){
+QHashIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
+int numeroFitxesActives=0;
+while (iterator_fitxes.hasNext()) {
+   iterator_fitxes.next();
+   if ( iterator_fitxes.value()->estat()==1){
+    ++numeroFitxesActives;}
+}
+bool finalDeJoc=false;
+    if( (numeroFitxesActives==1) &&  (p_tipusMoviment != 2)
+        && (comptadorFitxesFinalJoc==0) ){finalDeJoc=true;}
+    else if ((nomesQuedenFitxesMarcadesFinalJoc())
+             && (p_tipusMoviment != 2)){finalDeJoc=true;}
+    else if ( (numeroFitxesActives==numeroDeMovimentsJoc()+1) )
+            {finalDeJoc=true;}
+ return finalDeJoc;
+ }
+
  /*
    Per el moviment passat per paràmetre, comprova si hi ha
    moviments posteriors
-   */
+   Cal tenir en compte que pot ésser el darrer moviment!!!
+   19/11/12 Aquest proc. no pareix ésser eficaç. Ara no es fa
+   servir
+
  bool Tauler::solucionaJocActual_CalculaMovimentsPosteriors(QString moviment){
          int numeroMoviments=0;
          //Primer "simulam" el moviment
          QStringList fitxes=QStringList(moviment.split(" "));
-      if (p_tipusMoviment != 2){
+         if ( (p_tipusMoviment != 2) && (p_tipusMoviment != 5)){
          m_fitxes.value(fitxes.value(0).toInt())->setEstat(0);
          m_fitxes.value(fitxes.value(1).toInt())->setEstat(0);
          m_fitxes.value(fitxes.value(2).toInt())->setEstat(1);
          //Ara calculam el número de moviments possibles amb aquesta situació
          numeroMoviments=solucionaJocActual_MovimentsPosiblesActuals();
+
          //desfem el moviment
          m_fitxes.value(fitxes.value(0).toInt())->setEstat(1);
          m_fitxes.value(fitxes.value(1).toInt())->setEstat(1);
@@ -468,12 +736,16 @@ for(int j=0 ; j<itemsArbre5.count();++j){
          m_fitxes.value(fitxes.value(0).toInt())->setEstat(1);
          m_fitxes.value(fitxes.value(1).toInt())->setEstat(0);
          m_fitxes.value(fitxes.value(2).toInt())->setEstat(0);}
- return numeroMoviments>0;
+
+   //Comprovam si estem al final del joc
+  // if(arribatFinalJoc()){numeroMoviments=qMax(numeroMoviments,1);}
+
+     return numeroMoviments>0;
  }
+ */
 
  int Tauler::solucionaJocActual_MovimentsPosiblesActuals(){ 
     QHashIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
-    QStringList itemsArbre; //tots els moviments possibles
 
     int movimentsPosibles=0;
     int numeroFitxesActives=0;
@@ -494,11 +766,9 @@ for(int j=0 ; j<itemsArbre5.count();++j){
         }//if inicial estat==1
       }
 
- //Comprovam si estem al final del joc
-  //    qDebug("numeroDeMovimentsJoc()+1= %d",numeroDeMovimentsJoc()+1);
- if( (numeroFitxesActives==1) &&  (p_tipusMoviment != 2) ){movimentsPosibles=1;}
- else if ( (numeroFitxesActives==numeroDeMovimentsJoc()+1) ){movimentsPosibles=1;}
-  return movimentsPosibles;
+if (arribatFinalJoc()){movimentsPosibles=qMax(movimentsPosibles,1);}
+
+ return movimentsPosibles;
  }
 
  /*
@@ -506,9 +776,13 @@ for(int j=0 ; j<itemsArbre5.count();++j){
    o sigui fitxes en estat 1 que no tenen cap moviment
    que es pugui fer efectiu o be cap moviment en què
    la fitxa mitjancera pugui passar a estat 1 desprès d'un moviment
+   18/11/12 aques proc. no pareix molt eficaç
    */
  bool Tauler::solucionaJocActual_FitxesAillades(){
-
+     /* Aquest procediment no és eficient
+      * amb la modalitat inversa
+      */
+     if(p_tipusMoviment==2){return false;}
      int nivellAillamentFitxes=0;
      int numeroFitxesAillades=0;
      QHashIterator <int, Fitxa*> iterator_fitxes(m_fitxes);
@@ -518,9 +792,12 @@ for(int j=0 ; j<itemsArbre5.count();++j){
         int index_fitxaInicial=iterator_fitxes.value()->coordenades().x()*100+iterator_fitxes.value()->coordenades().y();
         moviments.clear();
         //Només poden estar "aïllades" les fitxes actives
+        //(si ho poden estar si son fitxes finals de joc!)
         //Tampoc pot estar aïllada una fitxa que és la mitjana d'un moviment posible
-        if( (m_fitxes.value(index_fitxaInicial)->estat()==1) &&
-            ( !(movimentPosibleFitxaMitjana(index_fitxaInicial))  )  ){
+        if( ((m_fitxes.value(index_fitxaInicial)->estat()==1) &&
+            ( !(movimentPosibleFitxaMitjana(index_fitxaInicial))  )) ||
+            (!(m_fitxes.value(index_fitxaInicial)->fitxaFinalJoc()) &&
+             ((m_fitxes.value(index_fitxaInicial)->estat()==1)))   ){
         for(int j=0 ; j<iterator_fitxes.value()->movimentsPossibles.count();++j){
             int index_fitxaBotada=iterator_fitxes.value()->movimentsPossibles.at(j).x();
             //fitxa activa amb fitxa mitjana en estat que permet el bot
@@ -551,6 +828,7 @@ for(int j=0 ; j<itemsArbre5.count();++j){
 return  (numeroFitxesAillades>p_numFitxesAillades);
  }
 
+
  /*
    Comprova si hi ha algun moviment posible que tengui com a fitxa final
    la fitxa passada per paràmetre
@@ -631,7 +909,7 @@ return  (numeroFitxesAillades>p_numFitxesAillades);
  }
 
  /*
-   No es da servir
+   No es fa servir
    Determina si la posició central
 
  bool Tauler::movimentFitxaCentral(){
@@ -667,20 +945,66 @@ return resultat;
      }
 }
 
- QString Tauler::configuracioActual(bool comptarFitxes){
+/* Posa totes les fitxes actives
+ * a color blau
+ */
+ void Tauler::eliminaFitxesVerdes(){
+    QHashIterator <int, Fitxa*> i(m_fitxes);
+    while (i.hasNext()) {
+        i.next();
+        if (i.value()->estat()==4){
+            i.value()->setEstat(1);
+        }
+     }
+}
+
+QString Tauler::configuracioActual(bool comptarFitxes){
      QHashIterator <int, Fitxa*> i(m_fitxes);
      QString configuracioActual;
-     if (comptarFitxes){comptadorFitxesJoc=0;}
+     if (comptarFitxes){
+         comptadorFitxesJoc=0;
+         comptadorFitxesEstatZeroJoc=0;
+         comptadorFitxesFinalJoc=0;
+     }
         while (i.hasNext()) {
               i.next();
               configuracioActual.append(QString("%1").arg((i.value()->estat())));
-              if ( (comptarFitxes) && (i.value()->estat()==1) ){++comptadorFitxesJoc;}
+              if (comptarFitxes){
+               if (i.value()->estat()==1){++comptadorFitxesJoc;}
+               if (i.value()->estat()==0){++comptadorFitxesEstatZeroJoc;}
+               if (i.value()->fitxaFinalJoc()){
+                  ++comptadorFitxesFinalJoc;
+                  }
+              }
           }
+/* Això cal quan es modifica un joc del programa
+ * i s'han eliminat fitxes.
+ */
+configuracioActual.remove("6");
+//qCritical("configuracioActual %s",qPrintable(configuracioActual));
 return configuracioActual;
  }
 
+/* Retorna la configuració actual de les fitxes amb les
+ * fitxes finals de joc marcades amb -
+ * 10/02/13 Al final no es fa servir
+
+QString Tauler::configuracioInicialAmbFitxesMarcades(){
+ QHashIterator <int, Fitxa*> i(m_fitxes);
+ QString configuracioActual;
+ while (i.hasNext()) {
+       i.next();
+       configuracioActual.append(QString("%1").arg((i.value()->estat())));
+       if (i.value()->fitxaFinalJoc()){
+           configuracioActual.append("-");
+       }
+   }
+return configuracioActual;
+}*/
+
+
  //Carrega la configuració passada
- void Tauler::carregaConfiguracio(QString configuracio){
+void Tauler::carregaConfiguracio(QString configuracio){
      QHashIterator <int, Fitxa*> i(m_fitxes);
      int contador=0;
      while (i.hasNext()) {
@@ -706,11 +1030,27 @@ void Tauler::estatFitxesAtzar(){
 
  /*
    Retorna el número de moviments que cal fer en el joc
+   (en funció del tipus de moviment)
    */
  int Tauler::numeroDeMovimentsJoc(){
-     if (p_tipusMoviment !=2){return comptadorFitxesJoc-1;}
+    /* qDebug("p_tipusMoviment %d",p_tipusMoviment);
+     qDebug("comptadorFitxesFinalJoc %d",comptadorFitxesFinalJoc);
+     qDebug("comptadorFitxesJoc %d",comptadorFitxesJoc);
+     qDebug("comptadorFitxesFinalJoc %d",comptadorFitxesFinalJoc);*/
+     if (p_tipusMoviment !=2){
+         if(comptadorFitxesFinalJoc<2){
+           return comptadorFitxesJoc-1;
+         }
+         else
+          return comptadorFitxesJoc-comptadorFitxesFinalJoc;
+      }
+       // qDebug("num, mov. %d",comptadorFitxesJoc-qMax(1,comptadorFitxesFinalJoc));
+        // return comptadorFitxesJoc-qMax(1,comptadorFitxesFinalJoc);}
      else {
-        return m_fitxes.count() - comptadorFitxesJoc-1;
+        // qDebug("m_fitxes.count() %d",m_fitxes.count());
+        // qDebug("comptadorFitxesJoc %d",comptadorFitxesJoc);
+        // 10/01/13 modificat
+        return comptadorFitxesEstatZeroJoc-1;
      }
  }
 
@@ -723,8 +1063,202 @@ void Tauler::estatFitxesAtzar(){
  }
  */
 
+
+ void Tauler::setModalitatJocActual(QString valor){
+   p_modalitatJocActual=valor;
+   setSolitariEspecial(p_modalitatJocActual);
+ }
+
  void Tauler::setEstatDelJoc(EstatsDelJoc valor){
      if(p_estatDelJoc != valor){
       p_estatDelJoc=valor;
     }
  }
+
+ /* Conserva el codi (per la traducció)
+  * del joc actual
+  */
+ void Tauler::setCodiNomJocActual (QString codi){
+     p_codiNomJocActual=codi;
+ }
+
+ /* En la generació d'un joc a l'atzar
+  * marca les fitxes que s'ha fet servir per
+  * generar el joc
+  */
+ void Tauler::solitariAtzar_MarcaFitxaUsada(int fitxa,bool marcaFitxaFinal){
+   m_fitxes.value(fitxa)->setfitxaUsadaGeneracioSolitariAtzar(true);
+   m_fitxes.value(fitxa)->setFitxaFinalJoc(marcaFitxaFinal);
+ }
+
+
+ /* En la generació d'un joc a l'atzar
+  * retorna l'estat de les fitxes del
+  * joc generat a l'atzar
+  */
+ QString Tauler::solitariAtzar_dadesEstatFitxes(int files, int columnes){
+     QHashIterator <int, Fitxa*> i(m_fitxes);
+     QString dadesFitxes;
+     QStringList dadesFitxesList;
+     //omplim la llista
+     for(int j=0; j<files*columnes; j++){
+         dadesFitxesList.append("-2");
+     }
+     i.toFront();
+     while (i.hasNext()) {
+              i.next();
+              //Si la fitxa està marcada (s'ha fet
+              //servir en la generació del joc)
+              if (i.value()->fitxaUsadaGeneracioSolitariAtzar()){
+                  //Si està marcada com a final del joc
+                  if(i.value()->fitxaFinalJoc()){
+                      dadesFitxesList.replace(
+                        i.value()->coordenades().x()*columnes+
+                        i.value()->coordenades().y(),
+                        QString("1%1").arg(i.value()->estat()));
+                  }
+                  else dadesFitxesList.replace(
+                         i.value()->coordenades().x()*columnes+
+                         i.value()->coordenades().y(),
+                         QString("%1").arg(i.value()->estat()));
+              }
+              else {dadesFitxesList.replace(
+                          i.value()->coordenades().x()*columnes+
+                          i.value()->coordenades().y(),
+                          QString("-1"));
+              }
+          }
+   dadesFitxes.clear();
+   //passam les dades de la llista a una cadena
+   for(int j=0; j<dadesFitxesList.count(); j++){
+       dadesFitxes=dadesFitxes+dadesFitxesList.value(j)+" ";
+   }
+   return dadesFitxes.simplified();
+ }
+
+ /* Estableix si s'està jugant un solitari a l'atzar
+  */
+ void Tauler::setSolitariAtzar(bool valor){
+  p_SolitariAtzar=valor;
+ }
+
+ /* Retorna les files i columnes, i les dades de les
+  * fitxes d'un solitari personalitzat
+  */
+ QStringList Tauler::solitariPersonalitzat_dadesFitxes(){
+ //calculem les files i columnes
+ QStringList filesColumnes=solitariPersonalitzat_filesColumnes().split(" ");
+ int minFila=filesColumnes.value(0).toInt();
+ int maxFila=filesColumnes.value(1).toInt();
+ int files=maxFila-minFila+1;
+ int minCol=filesColumnes.value(2).toInt();
+ int maxCol=filesColumnes.value(3).toInt();
+ int columnes=maxCol-minCol+1;
+
+ QStringList dadesFitxesList;
+ //omplim la llista
+ for(int j=0; j<files*columnes; j++){
+     dadesFitxesList.append("-1");
+ }
+ QHashIterator <int, Fitxa*> i(m_fitxes);
+ i.toFront();
+ while (i.hasNext()) {
+          i.next();
+          if( (i.value()->coordenades().x()>=minFila) &&
+              (i.value()->coordenades().x()<=maxFila) &&
+              (i.value()->coordenades().y()>=minCol) &&
+              (i.value()->coordenades().y()<=maxCol) ){
+              //els estats 6 no cal reemplaçar-los
+              //(ja hi ha -1 per defecte)
+              if (i.value()->estat() != 6){
+                 dadesFitxesList.replace(
+                     (i.value()->coordenades().x()-minFila)*columnes+
+                      i.value()->coordenades().y()-minCol,
+                      QString("%1").arg(i.value()->estat()));
+                 //Això per les fitxes marcades com a final de joc
+                 if(i.value()->fitxaFinalJoc()){
+                   dadesFitxesList.replace(
+                         (i.value()->coordenades().x()-minFila)*columnes+
+                          i.value()->coordenades().y()-minCol,
+                          QString("1%1").arg(i.value()->estat()));
+                 }
+              }
+          }
+    }//final while
+ //passam les dades de la llista a una cadena
+ QString dadesFitxes;
+ for(int j=0; j<dadesFitxesList.count(); j++){
+     dadesFitxes=dadesFitxes+dadesFitxesList.value(j)+" ";
+   }
+ dadesFitxesList.clear();
+ dadesFitxesList.append(QString("%1 %2").arg(files).arg(columnes));
+ dadesFitxesList.append(dadesFitxes.simplified());
+ //qCritical("Tauler::solitariPersonalitzat_dadesFitxes\n %s", qPrintable(dadesFitxes));
+ return dadesFitxesList;
+ }
+
+ /* Retorna el número de files i columnes del joc
+  * personalitzat.
+  * La llista té les coordenades més petita i més gran
+  * de files i columnes.
+  */
+ QString Tauler::solitariPersonalitzat_filesColumnes(){
+  QHashIterator <int, Fitxa*> i(m_fitxes);
+  int minFila=1000, maxFila=0,minCol=1000,maxCol=0;
+  while (i.hasNext()) {
+     i.next();
+     //Si la fitxa s'ha fet servir
+     if(i.value()->estat() != 6){
+         minFila=qMin(minFila,i.value()->coordenades().x()) ;
+         maxFila=qMax(maxFila,i.value()->coordenades().x()) ;
+         minCol=qMin(minCol,i.value()->coordenades().y()) ;
+         maxCol=qMax(maxCol,i.value()->coordenades().y()) ;
+     }
+  }
+ QString filesColumnes;
+ filesColumnes.append(QString("%1 %2 ").arg(minFila).arg(maxFila));
+ filesColumnes.append(QString("%1 %2").arg(minCol).arg(maxCol));
+ //QString missatge=QString(filesColumnes.value(0)+" "+filesColumnes.value(1));
+ //qCritical("%s", qPrintable(filesColumnes));
+ return filesColumnes;
+ }
+
+ /* Estableix si s'està jugant un solitari personalitzat
+  */
+ void Tauler::setSolitariPersonalitzat(bool valor){
+  p_SolitariPersonalitzat=valor;
+ }
+
+ /* Estableix si s'està jugant un solitari modificat
+  */
+ void Tauler::setSolitariModificat(bool valor){
+  p_SolitariModificat=valor;
+ }
+
+ /* Marcam totes les fitxes com a personalitzades
+  * (segons el valor passat)
+  */
+ void Tauler::marcaFitxesSolitariPersonalitzat(bool valor){
+     QHashIterator <int, Fitxa*> i(m_fitxes);
+     while (i.hasNext()) {
+        i.next();
+        i.value()->setFitxaSolitariPersonalitzat(valor);
+     }
+ }
+
+ /* Comprova si hi ha alguna fitxa del solitari marcada
+  * com a personalitzada
+  * 10/02/13 Finalment, no es fa servir això
+
+ bool Tauler::solitariPersonalitzat_hiHaFitxesMarcades(){
+  QHashIterator <int, Fitxa*> i(m_fitxes);
+  bool valor=false;
+  while (i.hasNext()) {
+    i.next();
+    if(i.value()->fitxaSolitariPersonalitzat()){
+      valor=true;
+      break;
+     }
+    }
+  return valor;
+ }*/
